--- Script para permitir que la IA deforme el suelo.
--- @author KCHARRO
--- @version 1.0 - Acivar Deformación del Suelo a la IA.

local AIGroundDeformation = {}

-- CONFIGURACIÓN AVANZADA (Ajusta estos valores según necesidad)
local SETTINGS = {
    DEBUG = false,                  -- Activar para ver mensajes de diagnóstico
    DEFORMATION_SCALE = 1,        -- Intensidad de deformación (1.0 = normal)
    FORCE_ALL_AI = true,            -- Forzar incluso en modos no estándar
    CHECK_INTERVAL = 150,           -- Cada 150ms (reducir si hay retraso)
    ENABLE_IMPLEMENT_DEFORM = true  -- Deformación para implementos
}

-- Sistema de traducción
local translations = {
    activation = {
        en = "[True AI Tracks] Script loaded: Ground deformation for AI activated!",
        es = "[Huellas de IA] Script cargado: ¡Defrmación del suelo para IA activado!",
        de = "[Echte KI-Tracks] Skript geladen: Bodenverformung für KI aktiviert!"
    },
    intensity = {
        en = "• Intensity: %.1f",
        es = "• Intensidad: %.1f",
        de = "• Intensität: %.1f"
    },
    monitoring = {
        en = "• Monitoring every: %dms",
        es = "• Monitoreo cada: %dms",
        de = "• Überwachung alle: %dms"
    },
    debugFormat = {
        en = "[True AI Tracks: DEFORM] %s: %s (Scale: %.1f)",
        es = "[Huellas de IA: DEFORM] %s: %s (Escala: %.1f)",
        de = "[Echte KI-Tracks: DEFORM] %s: %s (Skala: %.1f)"
    },
    debugStates = {
        en = { "OFF", "ON" },
        es = { "OFF", "ON" },  -- Se mantiene igual por ser términos técnicos
        de = { "AUS", "EIN" }
    }
}

-- Función para obtener mensaje traducido
local function getMessage(messageKey, ...)
    local lang = g_currentMission and g_currentMission.languageShort or "en"
    local translation = translations[messageKey][lang] or translations[messageKey]["en"]
    return string.format(translation, ...)
end

-- Variables internas
local lastCheckTime = 0
local terrainDeformationSystem = g_terrainDeformationSystem

-- Sobrescritura robusta de setDisplacementAllowed
local originalSetDisplacementAllowed = WheelPhysics.setDisplacementAllowed
function WheelPhysics:setDisplacementAllowed(displacementAllowed)
    -- Forzar configuración para IA
    if self.vehicle and self:isAIVehicle() then
        displacementAllowed = true
        self.displacementScale = SETTINGS.DEFORMATION_SCALE * 1.2  -- 20% extra para IA
    end

    -- Llamar a la función original con los valores modificados
    originalSetDisplacementAllowed(self, displacementAllowed)

    -- Debug detallado
    if SETTINGS.DEBUG and self.vehicle then
        local vehName = self.vehicle.getName and self.vehicle:getName() or "unknown"
        local states = translations.debugStates[g_currentMission and g_currentMission.languageShort or "en"] or translations.debugStates.en
        print(getMessage("debugFormat", 
              vehName, 
              displacementAllowed and states[2] or states[1], 
              self.displacementScale or 0))
    end
end

-- Función auxiliar para verificar IA
function WheelPhysics:isAIVehicle()
    if not self.vehicle then return false end
    return self.vehicle.isAI 
        or self.vehicle.spec_aiJobVehicle 
        or (self.vehicle.typeName and self.vehicle.typeName:match("AI"))
        or self.vehicle.cp  -- CoursePlay
        or self.vehicle.ad  -- AutoDrive
end

-- Verificación activa cada CHECK_INTERVAL ms
function AIGroundDeformation:update(dt)
    lastCheckTime = lastCheckTime + (dt or 0.016)
    if lastCheckTime < (SETTINGS.CHECK_INTERVAL / 1000) then return end
    lastCheckTime = 0

    for _, vehicle in ipairs(g_currentMission.vehicles or {}) do
        if vehicle:isAIVehicle() then
            self:processVehicle(vehicle)
        end
    end
end

-- Procesamiento completo del vehículo
function AIGroundDeformation:processVehicle(vehicle)
    -- Ruedas principales
    for _, wheel in ipairs(vehicle:getWheels() or {}) do
        if wheel.displacementAllowed ~= true then
            wheel.displacementAllowed = true
            wheel.displacementScale = SETTINGS.DEFORMATION_SCALE
        end
    end

    -- Implementos (opcional)
    if SETTINGS.ENABLE_IMPLEMENT_DEFORM then
        for _, implement in ipairs(vehicle:getAttachedImplements() or {}) do
            if implement.object and implement.object.setDisplacementAllowed then
                implement.object:setDisplacementAllowed(true)
            end
        end
    end
end

-- Inicialización limpia
function AIGroundDeformation:loadMap()
    print(getMessage("activation"))
end

addModEventListener(AIGroundDeformation)